# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2011 NTT.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Defines interface for DB access.

The underlying driver is loaded as a :class:`LazyPluggable`.
"""
from nova import flags
from nova import utils

FLAGS = flags.FLAGS
flags.DEFINE_string('ns_flat_db_backend', 
                    'sqlalchemy',
                    'Default backend to use for SQL.')

IMPL = utils.LazyPluggable(FLAGS['ns_flat_db_backend'],
                           sqlalchemy='nova.network.flat.db.sqlalchemy.api')

###################

def ethernet_card_get(context, id):
    """Get ethernet card by id."""
    return IMPL.dao_factory.get_dao(context).\
        ethernet_card_get(id)

def ethernet_card_create(context, values):
    """Create a new ethernet card."""
    return IMPL.dao_factory.get_dao(context).\
        ethernet_card_create(values)

def ethernet_card_get_all(context):
    """Get all ethernet cards."""
    return IMPL.dao_factory.get_dao(context).\
        ethernet_card_get_all()

def ethernet_card_update(context, ethernet_card_id, values):
    """Update ethernet card.""" 
    return IMPL.dao_factory.get_dao(context).\
        ethernet_card_update(ethernet_card_id, values)

def ethernet_card_delete(context, ethernet_card_id):
    """Delete ethernet card."""
    return IMPL.dao_factory.get_dao(context).\
        ethernet_card_delete(ethernet_card_id)

def network_get(context, id):
    """Get network by id."""
    return IMPL.dao_factory.get_dao(context).\
        network_get(id)

def network_get_by_bridge(context, bridge):    
    """Get a network by bridge or raise if it does not exist."""
    return IMPL.dao_factory.get_dao(context).\
        network_get_by_bridge(bridge)

def host_get_networks(context, host):
    """Get networks for a given host."""
    return IMPL.dao_factory.get_dao(context).\
        host_get_networks(host)
    
def network_create(context, values):
    """Create a new network."""
    return IMPL.dao_factory.get_dao(context).\
        network_create(values)

def network_create_safe(context, values):
    """Create a new network.  Returns None when there is an exception"""
    return IMPL.dao_factory.get_dao(context).\
        network_create_safe(values)

def network_get_all(context):
    """Get all networks."""
    return IMPL.dao_factory.get_dao(context).\
        network_get_all()

def network_update(context, network_id, values):
    """Update network.""" 
    return IMPL.dao_factory.get_dao(context).\
        network_update(network_id, values)

def network_delete(context, network_id):
    """Delete network."""
    return IMPL.dao_factory.get_dao(context).\
        network_delete(network_id)

def ip_address_get(context, id):
    """Get IP address by id."""
    return IMPL.dao_factory.get_dao(context).\
        ip_address_get(id)

def ip_address_get_by_ethernet_card(context, ethernet_card_id):
    """Get IP address by ethernet card id."""
    return IMPL.dao_factory.get_dao(context).\
        ip_address_get_by_ethernet_card(ethernet_card_id)

def ip_address_disassociate(context, address):
    """Disassocaites IP from ethernet card."""
    return IMPL.dao_factory.get_dao(context).\
        ip_address_disassociate(address)

def ip_address_get_by_address(self, address):
    """Get IP record by IP address."""
    return IMPL.dao_factory.get_dao(context).\
        ip_address_get_by_address(address)

def ip_address_create(context, values):
    """Create a new IP address."""
    return IMPL.dao_factory.get_dao(context).\
        ip_address_create(values)

def ip_address_get_all(context):
    """Get all IP addresses."""
    return IMPL.dao_factory.get_dao(context).\
        ip_address_get_all()

def ip_address_update(context, ip_address_id, values):
    """Update IP address.""" 
    return IMPL.dao_factory.get_dao(context).\
        ip_address_update(ip_address_id, values)

def ip_address_delete(context, ip_address_id):
    """Delete IP address."""
    return IMPL.dao_factory.get_dao(context).\
        ip_address_delete(ip_address_id)

def ip_address_associate_pool(context, network_id, ethernet_card_id):
    """Associate IP address to network and ethernet card.""" 
    return IMPL.dao_factory.get_dao(context).\
        ip_address_associate_pool(network_id, ethernet_card_id)
