# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2011 NTT.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from nova import context
from nova import flags
from nova import manager
from nova import utils

from nova.network.flat.db import manager as db_manager

FLAGS = flags.FLAGS
flags.DEFINE_string('ns_flat_network_driver', 'nova.network.linux_net',
                    'Driver to use for network creation')
flags.DEFINE_string('ns_flat_dns', '8.8.4.4',
                    'Dns for simple network')

class NetworkService(manager.Manager):

    def __init__(self, network_driver=None, *args, **kwargs):
        """Initializes flat network service."""
        if not network_driver:
            network_driver = FLAGS.ns_flat_network_driver
        self.driver = utils.import_object(network_driver)
        kwargs['db_driver'] = 'nova.network.flat.db.api' 
        super(NetworkService, self).__init__(*args, **kwargs)

    def init_host(self):
        """Do any initialization that needs to be run if this is a
        standalone service.
        """
        ctxt = context.get_admin_context()
        for network in self.db.host_get_networks(ctxt, self.host):
            self.set_network_host(ctxt, network['id'])

    def set_network_host(self, context, network_id):
        """Called when this host becomes the host for a network."""
        net = {}
        net['dns'] = FLAGS.ns_flat_dns
        net['host'] = self.host
        self.db.network_update(context, network_id, net)
        return self.host

    def allocate_fixed_ips(self, context, vnic_ids):
        """Gets a fixed ip from the pool."""
        for vnic_id in vnic_ids:
           db_manager.bind_ip_to_ethernet_card(context, vnic_id)

