# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from webob import exc

from nova.api.openstack import common as nova_common
from nova import exception
from nova import wsgi
from nova.api.openstack import faults
from nova.db import base
from nova.network.flat_dhcp import common
from nova.network.flat_dhcp import db
from nova.network.flat_dhcp.db import manager

class Controller(wsgi.Controller):
    """ The Virtual NICs API controller for the OpenStack API """
    
    def __init__(self):
        super(Controller, self).__init__()

    def index(self, req):
        """Return all ethernet cards in brief"""
        items = db.api.ethernet_card_get_all(req.environ['nova.context'])
        items = nova_common.limited(items, req)
        items = [common.filter_keys(item, ('id', 'mac_address')) 
                 for item in items]
        return dict(ethernet_cards=items)        
    
    def create(self, req):
        """ Creates a new ethernet card. """
        ethernet_card = manager.ethernet_card_create_with_random_mac(
                            req.environ['nova.context'])
        return dict(id=ethernet_card.id)
 
    def update(self, req, id):
        """Update an ethernet card"""
        return exc.HTTPNoContent()
    
    def show(self, req, id):
        """ Gets the ethernet card with ID given """
        try:
            ethernet_card = db.api.ethernet_card_get(
                                req.environ['nova.context'], id)
            return dict(id=ethernet_card.id,
                        mac_address=ethernet_card.mac_address)
        except exception.NotFound:
            return faults.Fault(exc.HTTPNotFound())

    def delete(self, req, id):
        """ Destroys an ethernet card """
        try:
            db.api.ethernet_card_delete(req.environ['nova.context'], id)
        except exception.NotFound:
            return faults.Fault(exc.HTTPNotFound())
        return exc.HTTPAccepted()
