# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2011 NTT.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Defines interface for DB access.

The underlying driver is loaded as a :class:`LazyPluggable`.
"""
from nova import flags
from nova import utils

FLAGS = flags.FLAGS
flags.DEFINE_string('ns_flat_dhcp_db_backend', 
                    'sqlalchemy',
                    'Default backend to use for SQL.')
flags.DEFINE_string('ns_flat_dhcp_sqlalchemy_api', 
                    'nova.network.flat_dhcp.db.sqlalchemy.ap',
                    'SQL Alchemy module to use.')

IMPL = utils.LazyPluggable(FLAGS['ns_flat_dhcp_db_backend'],
        sqlalchemy='nova.network.flat_dhcp.db.sqlalchemy.api')

###################

def ethernet_card_get(context, id):
    """Get ethernet card by id."""
    return IMPL.dao_factory.get_dao(context).\
        ethernet_card_get(id)

def ethernet_card_create(context, values):
    """Create a new ethernet card."""
    return IMPL.dao_factory.get_dao(context).\
        ethernet_card_create(values)

def ethernet_card_get_all(context):
    """Get all ethernet cards."""
    return IMPL.dao_factory.get_dao(context).\
        ethernet_card_get_all()

def ethernet_card_update(context, ethernet_card_id, values):
    """Update ethernet card.""" 
    return IMPL.dao_factory.get_dao(context).\
        ethernet_card_update(ethernet_card_id, values)

def ethernet_card_delete(context, ethernet_card_id):
    """Delete ethernet card."""
    return IMPL.dao_factory.get_dao(context).\
        ethernet_card_delete(ethernet_card_id)

def network_get(context, id):
    """Get network by id."""
    return IMPL.dao_factory.get_dao(context).\
        network_get(id)

def network_get_by_ethernet_card(context, ethernet_card_id):
    """Get network by ethernet card."""
    return IMPL.dao_factory.get_dao(context).\
        network_get_by_ethernet_card(ethernet_card_id)

def network_get_associated_fixed_ips(context, network_id):
    """Get fixed IPs that are associated with ethernet for a given network."""
    return IMPL.dao_factory.get_dao(context).\
        network_get_associated_fixed_ips(network_id)

def network_get_by_bridge(context, bridge):    
    """Get a network by bridge or raise if it does not exist."""
    return IMPL.dao_factory.get_dao(context).\
        network_get_by_bridge(bridge)
    
def host_get_networks(context, host):
    """Get networks for a given host."""
    return IMPL.dao_factory.get_dao(context).\
        host_get_networks(host)

def network_create(context, values):
    """Create a new network."""
    return IMPL.dao_factory.get_dao(context).\
        network_create(values)

def network_create_safe(context, values):
    """Create a new network.  Returns None when there is an exception"""
    return IMPL.dao_factory.get_dao(context).\
        network_create_safe(values)

def network_get_all(context):
    """Get all networks."""
    return IMPL.dao_factory.get_dao(context).\
        network_get_all()

def network_update(context, network_id, values):
    """Update network.""" 
    return IMPL.dao_factory.get_dao(context).\
        network_update(network_id, values)

def network_delete(context, network_id):
    """Delete network."""
    return IMPL.dao_factory.get_dao(context).\
        network_delete(network_id)

def fixed_ip_create(context, values):
    """Create a new fixed IP address."""
    return IMPL.dao_factory.get_dao(context).\
        fixed_ip_create(values)

def fixed_ip_get(context, fixed_ip_id):
    """Gets fixed IP with ID."""
    return IMPL.dao_factory.get_dao(context).\
        fixed_ip_get(fixed_ip_id)

def fixed_ip_get_by_ethernet_card(context, ethernet_card_id):
    """Get IP address by ethernet card id."""
    return IMPL.dao_factory.get_dao(context).\
        fixed_ip_get_by_ethernet_card(ethernet_card_id)

def fixed_ip_get_all(context):
    """Get all the fixed IP addresses."""
    return IMPL.dao_factory.get_dao(context).\
        fixed_ip_get_all()

def fixed_ip_update(context, fixed_ip_id, values):
    """Update fixed IP."""
    return IMPL.dao_factory.get_dao(context).\
        fixed_ip_update(fixed_ip_id, values)

def fixed_ip_get_by_address(context, address):
    """Get fixed IP by address."""
    return IMPL.dao_factory.get_dao(context).\
        fixed_ip_get_by_address(address)

def floating_ip_get_all_by_host(context, host):
    """Get all the floating IPs for a host."""
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_get_all_by_host(host)

def floating_ip_allocate_address(context, host, project_id):
    """Allocate free floating ip and return the address.

    Raises if one is not available.

    """
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_allocate_address(host, project_id)

def floating_ip_create(context, values):
    """Create a floating ip from the values dictionary."""
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_create(values)

def floating_ip_count_by_project(context, project_id):
    """Count floating ips used by project."""
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_count_by_project(project_id)

def floating_ip_deallocate(context, address):
    """Deallocate an floating ip by address"""
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_deallocate(address)

def floating_ip_destroy(context, address):
    """Destroy the floating_ip or raise if it does not exist."""
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_destroy(address)

def floating_ip_disassociate(context, address):
    """Disassociate an floating ip from a fixed ip by address.

    Returns the address of the existing fixed ip.

    """
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_disassociate(address)

def floating_ip_fixed_ip_associate(context, floating_address, fixed_address):
    """Associate an floating ip to a fixed_ip by address."""
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_fixed_ip_associate(floating_address, fixed_address)

def floating_ip_get_all(context):
    """Get all floating ips."""
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_get_all()

def floating_ip_get_all_by_project(context, project_id):
    """Get all floating ips by project."""
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_get_all_by_project(project_id)

def floating_ip_get_by_address(context, address):
    """Get a floating ip by address or raise if it doesn't exist."""
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_get_by_address(address)

def floating_ip_update(context, address, values):
    """Update a floating ip by address or raise if it doesn't exist."""
    return IMPL.dao_factory.get_dao(context).\
        floating_ip_update(address, values)

def fixed_ip_disassociate(context, fixed_ip_id):
    """Removes reference to ethernet card from IP."""
    return IMPL.dao_factory.get_dao(context).\
        fixed_ip_disassociate(fixed_ip_id)

def fixed_ip_associate_pool(context, network_id, ethernet_card_id):
    """Associates an IP to an ethernet card."""
    return IMPL.dao_factory.get_dao(context).\
        fixed_ip_associate_pool(network_id, ethernet_card_id)
