# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2011 NTT.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from nova import context
from nova import flags
from nova import log as logging
from nova import rpc

from nova.network.flat_dhcp.network import NetworkService

FLAGS = flags.FLAGS

LOG = logging.getLogger('nova.dhcpbridge-flat-dhcp')

class DHCPEventHandler(object):

    @classmethod
    def init_leases(cls, interface):
        """Get the list of hosts for an interface."""
        ctxt = context.get_admin_context()
        if FLAGS.fake_rabbit:
            LOG.debug(_("initializing leases"))
            return NetworkService().get_dhcp_host_leases(ctxt, interface)
	else:
            return rpc.call(ctxt,
                     "%s.%s" % (FLAGS.ns_flat_dhcp_network_topic, 
                                FLAGS.host),
                     {"method": "get_dhcp_host_leases",
                      "args": {"interface": interface}})

    @classmethod
    def add_lease(cls, mac, ip_address, _hostname, _interface):
        """Set the IP that was assigned by the DHCP server."""
        ctxt = context.get_admin_context()
        if FLAGS.fake_rabbit:
            LOG.debug(_("leasing ip"))
            return NetworkService().lease_fixed_ip(ctxt, mac, ip_address)
        else:
            rpc.cast(ctxt,
                     "%s.%s" % (FLAGS.ns_flat_dhcp_network_topic, 
                                FLAGS.host),
                     {"method": "lease_fixed_ip",
                      "args": {"mac": mac,
                               "address": ip_address}})

    @classmethod
    def old_lease(cls, mac, ip_address, hostname, interface):
        """Update just as add lease."""
        LOG.debug(_("Adopted old lease or got a change of mac/hostname"))
        cls.add_lease(mac, ip_address, hostname, interface)

    @classmethod
    def del_lease(cls, mac, ip_address, _hostname, _interface):
        """Called when a lease expires."""
        ctxt = context.get_admin_context()
        if FLAGS.fake_rabbit:
            LOG.debug(_("releasing ip"))
            NetworkService().release_fixed_ip(ctxt, mac, ip_address)
        else:
            rpc.cast(ctxt,
                     "%s.%s" % (FLAGS.ns_flat_dhcp_network_topic, 
                                FLAGS.host),
                     {"method": "release_fixed_ip",
                      "args": {"mac": mac,
                               "address": ip_address}})
    
