# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2011 NTT.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from zope import interface

from nova.network import service
from nova.network.vlan.db import api as db_api
from nova.network.vlan.db import manager as db_manager

class NetworkApiService(object):
    """Network API Service for this plugin."""
    
    interface.implements(service.INetworkApiService)

    def create_vnic(self, context):
        """Generic API to retrieve the default VNIC ID.
        For flat simple network, create a new vNIC and return its ID.
        
        Args:
           context: Nova context object needed to access the DB.

        Returns:
           The ID of the new VNIC.
        """
        # Create a new VNIC
        vnic = db_manager.ethernet_card_create_with_random_mac(context)
        return vnic.id
    
    def get_project_vpn_address_and_port(self, context, project_id):
        """Generic API to get network for a given project.
        
        Args:
            context: Nova context object needed to access the DB.
            project_id: project to get the VPN IP and port for.
           
        Returns:
           A tuple of VPN IP address and port number.
        """
        network = db_api.network_get_by_project(context, project_id)
        if network:
            return (network['vpn_public_address'], network['vpn_public_port'])
        else:
            return (None, None)

