# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from webob import exc

from nova import exception
from nova import wsgi
from nova.api.openstack import faults
from nova.db import base
from nova.network.vlan import common
from nova.network.vlan import db
from nova.network.vlan.db import manager

class Controller(wsgi.Controller):
    """ The Networks API controller for the OpenStack API """
    
    def __init__(self):
        super(Controller, self).__init__()

    def create(self, req):
        """ Creates a new network. """        
        env = self._deserialize(req.body, req.get_content_type())
        if not env:
            return faults.Fault(exc.HTTPUnprocessableEntity())

        networks = manager.network_create(req.environ['nova.context'], 
                                          env['cidr'], 
                                          int(env['num_networks']),
                                          int(env['network_size']),
                                          env['cidr_v6'],
                                          int(env['vlan_start']),
                                          int(env['vpn_start']))
        items = [common.filter_keys(item, ('id', 'cidr')) 
                 for item in networks] 
        return dict(ids=items)
    
    def update(self, req, id):
        """Updates a network."""
        env = self._deserialize(req.body, req.get_content_type())
        if not env:
            return faults.Fault(exc.HTTPUnprocessableEntity())
        
        # Currently only host can be updated.
        if 'host' in env:
            try:
                manager.network_update_host(req.environ['nova.context'], id, 
                                            env['host'])
            except exception.NotFound:
                return faults.Fault(exc.HTTPNotFound())

        return exc.HTTPNoContent()
    
    def show(self, req, id):
        """ Gets the network with ID given """
        try:
            network = db.api.network_get(req.environ['nova.context'], id)
            return dict(id=network.id, cidr=network.cidr)
        except exception.NotFound:
            return faults.Fault(exc.HTTPNotFound())

    def delete(self, req, id):
        """ Destroys a network """
        try:
            db.api.network_delete(req.environ['nova.context'], id)
        except exception.NotFound:
            return faults.Fault(exc.HTTPNotFound())
        return exc.HTTPAccepted()
