# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2011 NTT.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Implementation of SQLAlchemy backend.
"""

from sqlalchemy import or_
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm import joinedload_all
import warnings

from nova import exception
from nova.db import api as nova_db
from nova.db.sqlalchemy import api as nova_api
from nova.network.flat.db.sqlalchemy import api as flat_api
from nova.network.vlan.db.sqlalchemy import models
from nova.network.vlan.db.sqlalchemy.session import get_session


class DefaultSessionVlanNetworkDataAccessFactory(object):
    """A DAO factory that lazily uses the default session.

    This factory uses the default session for the VLAN network
    service, which it creates lazily when creating the first DAO.
    """

    def __init__(self):
        self._dao_cache = flat_api.DataAccessCache(
            models.VlanNetworkDataAccess)
        self._session = None

    def get_session(self):
        if self._session is None:
            self._session = get_session()
        return self._session

    def get_dao(self, context):
        """Get a DataAccess object.

        If no cached DAO has been created for this context's
        parameter, a new DAO is created and cached. Otherwise, the
        cached DAO is returned.

        :param context: The request context.
        """
        return self._dao_cache.get_dao(context, self.get_session())


dao_factory = DefaultSessionVlanNetworkDataAccessFactory()
