# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2011 NTT.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Session Handling for SQLAlchemy backend
"""

from sqlalchemy import create_engine
from sqlalchemy import pool
from sqlalchemy.orm import sessionmaker

from nova import exception
from nova import flags

FLAGS = flags.FLAGS
flags.DEFINE_integer('ns_vlan_sql_idle_timeout',
                     3600,
                     'timeout for idle sql database connections')
flags.DEFINE_string('ns_vlan_sql_connection',
              'sqlite:///$state_path/nova_vlan_network.sqlite',
              'connection string for VLAN network sql database')

_VLAN_NETWORK_ENGINE = None
_VLAN_NETWORK_MAKER = None

def get_session(autocommit=True, expire_on_commit=False):
    """Helper method to grab session"""
    global _VLAN_NETWORK_ENGINE
    global _VLAN_NETWORK_MAKER
    if not _VLAN_NETWORK_MAKER:
        if not _VLAN_NETWORK_ENGINE:
            kwargs = {'pool_recycle': FLAGS.ns_vlan_sql_idle_timeout,
                      'echo': False}

            if FLAGS.ns_vlan_sql_connection.startswith('sqlite'):
                kwargs['poolclass'] = pool.NullPool

            _VLAN_NETWORK_ENGINE = create_engine(
                                    FLAGS.ns_vlan_sql_connection,
                                    **kwargs)            
            
        _VLAN_NETWORK_MAKER = (sessionmaker(
                               bind=_VLAN_NETWORK_ENGINE,
                               autocommit=autocommit,
                               expire_on_commit=expire_on_commit))

    session = _VLAN_NETWORK_MAKER()
    session.query = exception.wrap_db_error(session.query)
    session.flush = exception.wrap_db_error(session.flush)
    return session
